"""Utility functions"""

import shlex
from configparser import ConfigParser
from subprocess import check_output, check_call, CalledProcessError


def parse_list_option(opt):
    """Split list element separated by a comma."""
    if not opt:
        return []
    return sorted(opt.split(','))


def _get_git_config(config):
    cmd = 'git config --get {config}'.format(config=config)
    try:
        config = check_output(shlex.split(cmd)).decode()[:-1]
    except CalledProcessError:
        config = ''

    return config


def get_username():
    return _get_git_config('user.name')


def get_usermail():
    return _get_git_config('user.email')


def clone_repository(url, version, dest):
    cmd = 'git clone --depth=1 -b {version} {url} {dest}'.format(
        dest=dest, url=url, version=version
    )
    return check_call(shlex.split(cmd))


def read_config(filename):
    parser = ConfigParser()
    parser.readfp(filename)
    return parser._sections



"""RIOT pkg generator module."""

import os

import click

from .common import render_source, render_file
from .common import check_param, prompt_param, prompt_param_list
from .common import check_riotbase
from .utils import read_config, parse_list_option


def read_pkg_config(filename):
    """Read the pkg specific configuration file."""
    params = read_config(filename)
    _params = params['pkg']
    for param in ['modules', 'packages', 'features']:
        _params[param] = parse_list_option(_params[param])
    return params


def prompt_pkg_params(params):
    """Request pkg specific variables."""
    _params = params['pkg']
    prompt_param(_params, 'name', 'Package name')
    prompt_param(
        _params, 'displayed_name',
        'Package displayed name (for doxygen documentation)')
    prompt_param(_params, 'url', 'Package source url')
    prompt_param(_params, 'hash', 'Package version hash')
    prompt_param(_params, 'license', 'Package license')
    prompt_param(_params, 'description', 'Package short description')
    prompt_param_list(
        _params, 'modules', 'Required modules (comma separated)')
    prompt_param_list(
        _params, 'packages', 'Required packages (comma separated)')
    prompt_param_list(
        _params, 'features', 'Required features (comma separated)')


def check_pkg_params(params):
    _params = params['pkg']
    for param in ['name', 'displayed_name', 'url', 'hash', 'license']:
        check_param(_params, param)
    _params['name'] = _params['name'].replace(' ', '_')


def generate_pkg(interactive, config, riotbase):
    if not interactive and config is None:
        raise click.MissingParameter(
            param_type='--interactive and/or --config options'
        )

    check_riotbase(riotbase)

    params = {
        'pkg': {}
    }

    if config is not None:
        params = read_pkg_config(config)

    if interactive:
        prompt_pkg_params(params)

    check_pkg_params(params)

    _params = params['pkg']

    riotbase = os.path.abspath(os.path.expanduser(riotbase))
    pkgs_dir = os.path.join(riotbase, 'pkg')
    pkg_dir = os.path.join(pkgs_dir, _params['name'])

    if os.path.abspath(os.path.curdir) == riotbase:
        output_dir = os.path.join('pkg', _params['name'])
    else:
        output_dir = os.path.expanduser(pkg_dir)

    if not os.path.exists(pkg_dir):
        os.makedirs(pkg_dir)
    elif not click.prompt('\'{name}\' pkg directory already exists, '
                          'overwrite (y/N)?'.format(_params['name']),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    render_source(
        params, 'pkg',
        ['doc.txt', 'Makefile', 'Makefile.dep', 'Makefile.include'],
        output_dir
    )

    template_dir = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), 'templates', 'pkg'
    )
    makefile_pkg_out = os.path.join(output_dir,
                                    '{}.mk'.format(_params['name']))
    render_file(params, template_dir, 'pkg.mk.j2', makefile_pkg_out)

    click.echo(click.style(
        'Package \'{name}\' generated in {output_dir} with success!'
        .format(name=_params['name'], output_dir=output_dir), bold=True))



"""Common generator module."""

import os
import datetime

from jinja2 import Environment, FileSystemLoader
from click import prompt, MissingParameter

from .utils import get_usermail, get_username, parse_list_option


def check_riotbase(riotbase):
    """Check the given path is a valid RIOTBASE directory."""
    if riotbase is None or not riotbase:
        raise MissingParameter(param_type='riotbase directory')


def check_common_params(params):
    _params = params['common']
    if 'year' not in _params or not _params['year']:
        _params['year'] = datetime.datetime.now().year
    if 'author_name' not in _params or not _params['author_name']:
        _params['author_name'] = get_username()
    if 'author_email' not in _params or not _params['author_email']:
        _params['author_email'] = get_usermail()
    if 'organization' not in _params or not _params['organization']:
        _params['organization'] = get_username()
    check_param(_params, 'author_name')
    check_param(_params, 'author_email')
    check_param(_params, 'organization')


def check_param(params, param):
    if param not in params or params[param] == '':
        raise MissingParameter(param_type=param.replace('_', ' '))


def prompt_param(params, param, text, default=None, show_default=True):
    if param not in params or not params[param]:
        params[param] = prompt(
            text=text, default=default, show_default=show_default
        )


def prompt_param_list(params, param, text):
    if param not in params or not params[param]:
        params[param] = prompt(
            text=text, default='', value_proc=parse_list_option
        )


def prompt_common_params(params):
    _params = params['common']
    if 'year' not in params or not _params['year']:
        _params['year'] = datetime.datetime.now().year
    prompt_param(_params, 'author_name', 'Author name', default=get_username())
    prompt_param(
        _params, 'author_email', 'Author email', default=get_usermail())
    prompt_param(
        _params, 'organization', 'Organization', default=get_username())


def render_file(context, template_dir, source, dest):
    """Generate a file from an input template and a dict of parameters."""
    loader = FileSystemLoader(searchpath=template_dir)
    env = Environment(loader=loader,
                      trim_blocks=True,
                      lstrip_blocks=True,
                      keep_trailing_newline=True)
    env.globals.update(zip=zip)
    template = env.get_template(source)
    render = template.render(**context)
    with open(dest, 'w') as f_dest:
        f_dest.write(render)


def render_source(context, template_dir, input_files, output_dir,
                  output_subdir=""):
    """Generate a list of files given from an input template directory."""
    template_dir = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), 'templates', template_dir
    )
    if output_subdir:
        output_dir = os.path.join(output_dir, output_subdir)

    files = {
        filename + '.j2': os.path.join(output_dir, filename)
        for filename in input_files
    }

    for source, dest in files.items():
        render_file(context, template_dir, source, dest)



"""RIOT application generator module."""

import os

import click

from .common import render_source
from .common import check_common_params, check_param, check_riotbase
from .common import prompt_common_params, prompt_param, prompt_param_list
from .utils import read_config, parse_list_option


def read_application_config(filename):
    """Read the application specific configuration file."""
    params = read_config(filename)
    _params = params['application']
    for param in ['modules', 'packages', 'features']:
        if param not in _params:
            _params[param] = []
        else:
            _params[param] = parse_list_option(_params[param])
    return params


def prompt_application_params(params):
    """Request application specific variables."""
    _params = params['application']
    prompt_param(_params, 'name', 'Application name')
    prompt_param(_params, 'brief', 'Application brief description')
    prompt_param(_params, 'board', 'Target board', default='native')
    prompt_param_list(
        _params, 'modules', 'Required modules (comma separated)')
    prompt_param_list(
        _params, 'packages', 'Required packages (comma separated)')
    prompt_param_list(
        _params, 'features', 'Required features (comma separated)')


def check_application_params(params):
    _params = params['application']
    for param in ['name', 'board', 'brief']:
        check_param(_params, param)
    _params['name'] = _params['name'].replace(' ', '_')


def generate_application(output_dir, interactive, config, riotbase):
    if not interactive and config is None:
        raise click.MissingParameter(
            param_type='--interactive and/or --config options'
        )

    check_riotbase(riotbase)

    params = {
        'common': {},
        'application': {
            'riotbase': os.path.abspath(os.path.expanduser(riotbase))
        }
    }

    if config is not None:
        params = read_application_config(config)

    if interactive:
        prompt_application_params(params)
        prompt_common_params(params)

    check_application_params(params)
    check_common_params(params)

    output_dir = os.path.expanduser(output_dir)
    render_source(
        params,
        'application',
        ['main.c', 'Makefile', 'README.md'],
        output_dir
    )

    click.echo(click.style(
        'Application \'{name}\' generated in {output_dir} with success!'
        .format(name=params['application']['name'], output_dir=output_dir),
        bold=True
    ))
    click.echo('\nTo build the application, use')
    click.echo('\n     make -C {output_dir}\n'.format(output_dir=output_dir))



import os
import click

from .application import generate_application
from .example import generate_example
from .board import generate_board
from .pkg import generate_pkg
from .test import generate_test


@click.group()
def cli():
    pass


@cli.command(help='Bootstrap a RIOT application')
@click.option(
    '-d', '--output_dir', type=click.Path(exists=True), default=os.getcwd())
@click.option('-i', '--interactive', is_flag=True, help='Use interactive mode')
@click.option('--config', type=click.File(mode='r'),
              help='Configuration file for application')
@click.option(
    '-r', '--riotbase', type=click.Path(exists=True),
    default=os.getenv('RIOTBASE'))
def application(output_dir, interactive, config, riotbase):
    generate_application(output_dir, interactive, config, riotbase)


@cli.command(help='Bootstrap a RIOT board support')
@click.option('-i', '--interactive', is_flag=True, help='Use interactive mode')
@click.option('--config', type=click.File(mode='r'),
              help='Configuration file for board')
@click.option(
    '-r', '--riotbase', type=click.Path(exists=True),
    default=os.getenv('RIOTBASE'))
def board(interactive, config, riotbase):
    generate_board(interactive, config, riotbase)


@cli.command(help='Bootstrap a RIOT example application')
@click.option('-i', '--interactive', is_flag=True, help='Use interactive mode')
@click.option('--config', type=click.File(mode='r'),
              help='Configuration file for example application')
@click.option(
    '-r', '--riotbase', type=click.Path(exists=True),
    default=os.getenv('RIOTBASE'))
def example(interactive, config, riotbase):
    generate_example(interactive, config, riotbase)


@cli.command(help='Bootstrap a RIOT external package')
@click.option('-i', '--interactive', is_flag=True, help='Use interactive mode')
@click.option('--config', type=click.File(mode='r'),
              help='Configuration file for package')
@click.option(
    '-r', '--riotbase', type=click.Path(exists=True),
    default=os.getenv('RIOTBASE'))
def pkg(interactive, config, riotbase):
    generate_pkg(interactive, config, riotbase)


@cli.command(help='Bootstrap a RIOT test application')
@click.option('-i', '--interactive', is_flag=True, help='Use interactive mode')
@click.option('--config', type=click.File(mode='r'),
              help='Configuration file for test application')
@click.option(
    '-r', '--riotbase', type=click.Path(exists=True),
    default=os.getenv('RIOTBASE'))
def test(interactive, config, riotbase):
    generate_test(interactive, config, riotbase)



"""RIOT application generator module."""

import os

import click

from .common import render_source
from .common import check_common_params, check_param, check_riotbase
from .common import prompt_common_params, prompt_param, prompt_param_list
from .utils import read_config, parse_list_option


def read_board_config(filename):
    """Read the board specific configuration file."""
    params = read_config(filename)
    _params = params['board']
    if 'features' not in _params:
        _params['features'] = []
    else:
        _params['features'] = parse_list_option(_params['features'])
    return params


def prompt_board_params(params):
    """Request board specific variables."""
    _params = params['board']
    prompt_param(_params, 'name', 'Board name')
    prompt_param(
        _params, 'displayed_name',
        'Board displayed name (for doxygen documentation)')
    prompt_param(_params, 'cpu', 'CPU name')
    prompt_param(_params, 'cpu_model', 'CPU model name')
    prompt_param_list(
        _params,
        'features',
        'Features provided by this board (comma separated)'
    )


def check_board_params(params):
    _params = params['board']
    for param in ['name', 'cpu', 'cpu_model', 'displayed_name']:
        check_param(_params, param)
    _params['name'] = _params['name'].replace(' ', '_')


def generate_board(interactive, config, riotbase):
    if not interactive and config is None:
        raise click.MissingParameter(
            param_type='--interactive and/or --config options'
        )

    check_riotbase(riotbase)

    params = {
        'common': {},
        'board': {}
    }

    if config is not None:
        params = read_board_config(config)

    if interactive:
        prompt_board_params(params)
        prompt_common_params(params)

    check_board_params(params)
    check_common_params(params)

    _params = params['board']
    riotbase = os.path.abspath(os.path.expanduser(riotbase))
    boards_dir = os.path.join(riotbase, 'boards')
    board_dir = os.path.join(boards_dir, _params['name'])
    board_include_dir = os.path.join(board_dir, 'include')

    if not os.path.exists(board_dir):
        os.makedirs(board_dir)
        os.makedirs(board_include_dir)
    elif not click.prompt('\'{name}\' board directory already exists, '
                          'overwrite (y/N)?'.format(_params['name']),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    render_source(
        params, 'board',
        [
            'board.c', 'doc.txt', 'Makefile', 'Makefile.dep',
            'Makefile.features', 'Makefile.include'
        ],
        board_dir
    )

    render_source(
        params, 'board',
        ['board.h', 'periph_conf.h'],
        board_dir, output_subdir='include'
    )

    click.echo(click.style(
        'Support for board \'{board}\' generated!'.format(
            board=_params['displayed_name']
        ),
        bold=True))



"""RIOT test generator module."""

import os

import click

from .common import render_source
from .common import check_common_params, check_param, check_riotbase
from .common import prompt_common_params, prompt_param, prompt_param_list
from .utils import read_config, parse_list_option


def read_test_config(filename):
    """Read the test specific configuration file."""
    params = read_config(filename)
    _params = params['test']
    for param in ['modules', 'packages', 'features']:
        _params[param] = parse_list_option(_params[param])
    return params


def prompt_test_params(params):
    """Request test specific variables."""
    _params = params['test']
    prompt_param(_params, 'name', 'Test name')
    prompt_param(_params, 'brief', 'Test brief description')
    prompt_param(_params, 'board', 'Target board', default='native')
    prompt_param_list(
        _params, 'modules', 'Required modules (comma separated)')
    prompt_param_list(
        _params, 'packages', 'Required packages (comma separated)')
    prompt_param_list(
        _params, 'features', 'Required features (comma separated)')
    prompt_param(
        _params, 'use_testrunner', 'Add testrunner script (y/N)?',
        default=False, show_default=False)


def check_test_params(params):
    _params = params['test']
    for param in ['name', 'board', 'brief', 'use_testrunner']:
        check_param(_params, param)
    _params['name'] = _params['name'].replace(' ', '_')


def generate_test(interactive, config, riotbase):
    if not interactive and config is None:
        raise click.MissingParameter(
            param_type='--interactive and/or --config options'
        )

    check_riotbase(riotbase)

    params = {
        'common': {},
        'test': {}
    }

    if config is not None:
        params = read_test_config(config)

    if interactive:
        prompt_test_params(params)
        prompt_common_params(params)

    check_test_params(params)
    check_common_params(params)
    _params = params['test']

    riotbase = os.path.abspath(os.path.expanduser(riotbase))
    tests_dir = os.path.join(riotbase, 'tests')
    test_dir = os.path.join(tests_dir, _params['name'])

    if os.path.abspath(os.path.curdir) == riotbase:
        output_dir = os.path.join('tests', _params['name'])
    else:
        output_dir = os.path.expanduser(test_dir)

    if not os.path.exists(test_dir):
        os.makedirs(test_dir)
    elif not click.prompt('\'{name}\' test directory already exists, '
                          'overwrite (y/N)?'.format(name=_params['name']),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    if 'use_testrunner' in _params and _params['use_testrunner'] == "True":
        testrunner_dir = os.path.join(test_dir, 'tests')
        if not os.path.exists(testrunner_dir):
            os.makedirs(testrunner_dir)
        render_source(params, 'test', ['01-run.py'], testrunner_dir)
        os.chmod(os.path.join(testrunner_dir, '01-run.py'), 0o755)

    files = ['main.c', 'Makefile', 'README.md']
    render_source(params, 'test', files, output_dir)

    click.echo(click.style(
        'Test application \'{name}\' generated in {output_dir} with success!'
        .format(name=_params['name'], output_dir=output_dir), bold=True))



"""RIOT generator is a helper library for bootstraping RIOT code.

It can be used to bootstrap:
- a new board support
- a new external application
- a new RIOT example application
- a new test application
- a new package for RIOT
"""

__version__ = '0.3.0'

from .generator import cli  # noqa



#!/usr/bin/env python

from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path

PACKAGE = 'riotgen'


def get_long_description():
    # Get the long description from the README file
    with open('README.rst', encoding='utf-8') as f:
        return f.read()


def get_version():
    """Get the version from package __init__.py file."""
    with open(path.join(PACKAGE, '__init__.py'), encoding='utf-8') as f:
        for line in f:
            if line.startswith('__version__'):
                return eval(line.split('=')[-1])


setup(
    name=PACKAGE,
    version=get_version(),

    description='riotgen: generator for RIOT application/driver/board code',
    long_description=get_long_description(),
    long_description_content_type='text/x-rst',

    url='https://github.com/aabadie/riot-generator',

    author='Alexandre Abadie',
    author_email='alexandre.abadie@inria.fr',

    license='',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],

    keywords='generator code',
    platforms='any',
    packages=[PACKAGE],
    include_package_data=True,
    package_dir={
        'riotgen': 'riotgen'
    },
    package_data={
        'riotgen': [
            'templates/*/*',
        ]
    },
    install_requires=[
        'click',
        'Jinja2',
    ],
    entry_points={
        'console_scripts': [
            '{}=riotgen:cli'.format(PACKAGE),
        ],
    },
)
